package org.ow2.orchestra.definition.activity;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.CompositeElement;
import org.ow2.orchestra.facade.data.def.ProcessActivityDefinitionData;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.BpelInstance;
import org.ow2.orchestra.services.MessageCarrierRepository;
import org.ow2.orchestra.services.handlers.FinishedInstanceHandler;
import org.ow2.orchestra.services.itf.Replier;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.Misc;

public class Process extends Scope {

  private static final long serialVersionUID = -375673090384319392L;

  private static final Logger LOG = Logger.getLogger(Process.class.getName());

  public Process() {
    super();
  }

  @Override
  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      super.signal(execution, null, parameters);
      return;
    }
    // execute source to log end of scope ?
    this.afterRunned(execution);
    if (execution.isActive()) {
      final CompositeElement parent = execution.getNode().getParent();
      if (parent == null) {
        Misc.fastDynamicLog(Process.LOG, Level.FINE,"End of bpel process execution");

        if (execution.getParent() != null) {
          throw new IllegalStateException("Child execution reaches the end of process !");
        }
        if (!execution.searchWaitingExecutions().isEmpty()) {
          throw new IllegalStateException(
            "Execution reaches the end of process, but child executions are still waiting for a trigger !");
        }
        execution.removeJobs();
        execution.end();

      }
      // Call the FinishedInstanceHandler
      Process.endProcessInstance(execution);
    }
  }

  public static void endProcessInstance(final BpelExecution execution) {
    final BpelInstance instance = execution.getProcessInstance();
    // close pending connections
    for (final Replier replier : instance.getRepliers()) {
      MessageCarrierRepository.throwMissingReply(replier.getReplierKey());
    }
    // Call the FinishedInstanceHandler
    final FinishedInstanceHandler handler = EnvTool.get(FinishedInstanceHandler.class, FinishedInstanceHandler.DEFAULT_KEY);
    if (handler != null) {
      handler.handleFinishedInstance(instance.getProcessInstanceUUID());
    }

  }

  @Override
  public ProcessActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new ProcessActivityDefinitionData(bpelProcess.getProcessDefinitionData(), this.uuid, null, this.name, null);
  }

  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {
    final BpelInstance bpelInstance = bpelExecution.getProcessInstance();
    final ActivityInstanceUUID activityInstanceUUID = bpelInstance.getProcessInstanceUUID();
    bpelExecution.setActivityInstanceUUID(activityInstanceUUID);
    bpelInstance.setProcessInstanceState(ActivityState.RUNNING);

    EnvTool.getRecorder().recordInstanceStarted(bpelInstance.getProcessInstanceData());

  }

  @Override
  public void afterRunned(final BpelExecution bpelExecution) {
    final BpelInstance bpelInstance = bpelExecution.getProcessInstance();
    bpelInstance.setProcessInstanceState(ActivityState.FINISHED);
    EnvTool.getRecorder().recordInstanceEnded(bpelInstance.getProcessInstanceData());
  }
}
