/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.env.binding;

import java.util.Collection;
import java.util.List;

import org.ow2.orchestra.cluster.StaticClusterDescription;
import org.ow2.orchestra.env.descriptor.ArgDescriptor;
import org.ow2.orchestra.env.descriptor.CollectionDescriptor;
import org.ow2.orchestra.env.descriptor.ListDescriptor;
import org.ow2.orchestra.env.descriptor.ObjectDescriptor;
import org.ow2.orchestra.xml.Parse;
import org.ow2.orchestra.xml.Parser;
import org.w3c.dom.Element;


/**
 * @author Guillaume Porcher
 *
 */
public class StaticClusterDescriptionBinding extends AbstractCollectionBinding {

  public StaticClusterDescriptionBinding() {
    super("static-cluster");
  }
  @Override
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    parser.importExternalReferences(element, parse);

    final CollectionDescriptor collectionDescriptor = (CollectionDescriptor) super.parse(element, parse, parser);

    final ObjectDescriptor clusterDescriptor = new ObjectDescriptor(StaticClusterDescription.class);

    final ArgDescriptor argDescriptor = new ArgDescriptor();
    argDescriptor.setTypeName(Collection.class.getName());
    argDescriptor.setDescriptor(collectionDescriptor);
    clusterDescriptor.addArgDescriptor(argDescriptor);

    return clusterDescriptor;
  }

  @Override
  protected CollectionDescriptor createDescriptor() {
    return new ListDescriptor();
  }

  @Override
  protected Class< ? > getCollectionInterface() {
    return List.class;
  }
}
