package org.ow2.orchestra.env.descriptor;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.env.Descriptor;
import org.ow2.orchestra.env.WireContext;
import org.ow2.orchestra.env.WireException;
import org.ow2.orchestra.util.Misc;
import org.ow2.orchestra.util.ReflectUtil;

/**
 * @author Tom Baeyens
 */
public class CollectionDescriptor extends AbstractDescriptor implements
    Descriptor {

  private static final long serialVersionUID = 1L;
  private static final Logger LOG = Logger.getLogger(CollectionDescriptor.class.getName());

  protected String className;
  protected List<Descriptor> valueDescriptors;
  protected boolean isSynchronized;

  private transient Class< ? > clazz;

  protected CollectionDescriptor() {
  }

  public CollectionDescriptor(final String defaultImplClassName) {
    this.className = defaultImplClassName;
  }

  public Object construct(final WireContext wireContext) {
    Object object = null;
    try {
      // instantiate
      if (this.clazz == null) {
        final ClassLoader classLoader = wireContext.getClassLoader();
        this.clazz = ReflectUtil.loadClass(classLoader, this.className);
      }
      object = this.clazz.newInstance();

      if (this.isSynchronized) {
        if (object instanceof SortedSet) {
          object = Collections.synchronizedSortedSet((SortedSet) object);
        } else if (object instanceof SortedMap) {
          object = Collections.synchronizedSortedMap((SortedMap) object);
        } else if (object instanceof Set) {
          object = Collections.synchronizedSet((Set) object);
        } else if (object instanceof Map) {
          object = Collections.synchronizedMap((Map) object);
        } else if (object instanceof List) {
          object = Collections.synchronizedList((List) object);
        } else if (object instanceof Collection) {
          object = Collections.synchronizedCollection((Collection) object);
        }
      }

    } catch (final Exception e) {
      throw new WireException("couldn't create '"
          + (this.name != null ? this.name : this.className) + "': " + e.getMessage(), e);
    }
    return object;
  }

  @Override
  public void initialize(final Object object, final WireContext wireContext) {
    final Collection<Object> collection = (Collection<Object>) object;
    try {
      if (this.valueDescriptors != null) {
        for (final Descriptor descriptor : this.valueDescriptors) {
          final Object element = wireContext.create(descriptor, true);
          Misc.fastDynamicLog(CollectionDescriptor.LOG, Level.FINEST, "adding element %s to collection", element);
          collection.add(element);
        }
      }
    } catch (final WireException e) {
      throw e;
    } catch (final Exception e) {
      throw new WireException("couldn't initialize object '"
          + (this.name != null ? this.name : this.className) + "'", e);
    }
  }

  public String getClassName() {
    return this.className;
  }

  public void setClassName(final String className) {
    this.className = className;
  }

  public List<Descriptor> getValueDescriptors() {
    return this.valueDescriptors;
  }

  public void setValueDescriptors(final List<Descriptor> valueDescriptors) {
    this.valueDescriptors = valueDescriptors;
  }

  public boolean isSynchronized() {
    return this.isSynchronized;
  }

  public void setSynchronized(final boolean isSynchronized) {
    this.isSynchronized = isSynchronized;
  }
}
