/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.util.List;
import java.util.Set;

import javax.xml.namespace.QName;

import org.ow2.orchestra.env.EnvironmentFactory;
import org.ow2.orchestra.facade.QuerierDefinitionAPI;
import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.ProcessDefinition;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.exception.ActivityNotFoundException;
import org.ow2.orchestra.facade.exception.OrchestraWrapperException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.jmx.commands.FindActivityDefinitionsCommand;
import org.ow2.orchestra.jmx.commands.FindProcessDefinitionsCommand;
import org.ow2.orchestra.jmx.commands.GetActivityDefinitionCommand;
import org.ow2.orchestra.jmx.commands.GetProcessDefinitionCommand;
import org.ow2.orchestra.services.commands.CommandService;

public class QuerierDefinitionAPIImpl implements QuerierDefinitionAPI {

  private final EnvironmentFactory environmentFactory;

  public QuerierDefinitionAPIImpl(final EnvironmentFactory environmentFactory) {
    this.environmentFactory = environmentFactory;
  }

  protected CommandService getCommandService() {
    return this.environmentFactory.get(CommandService.class);
  }

  public List<ProcessDefinition> findProcessDefinitions() {
    return this.getCommandService().execute(new FindProcessDefinitionsCommand());
  }

  public List<ProcessDefinition> findProcessDefinitions(final ProcessState state) {
    return this.getCommandService().execute(new FindProcessDefinitionsCommand(state));
  }

  public List<ProcessDefinition> findProcessDefinitions(final QName processName, final ProcessState state) {
    return this.getCommandService().execute(new FindProcessDefinitionsCommand(state, processName));
  }

  public List<ProcessDefinition> findProcessDefinitions(final QName processName) {
    return this.getCommandService().execute(new FindProcessDefinitionsCommand(processName));
  }

  public ProcessDefinition getProcessDefinition(final ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new GetProcessDefinitionCommand(processUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public ActivityDefinition getActivityDefinition(final ActivityDefinitionUUID activityDefUUID) throws ActivityNotFoundException {
    try {
      return this.getCommandService().execute(new GetActivityDefinitionCommand(activityDefUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ActivityNotFoundException) {
        throw (ActivityNotFoundException) e.getCause();
      }
      throw e;
    }
  }

  public Set<ActivityDefinition> findActivityDefinitions(final ProcessDefinitionUUID processUUID)
    throws ProcessNotFoundException {
    try {
      return this.getCommandService().execute(new FindActivityDefinitionsCommand(processUUID));
    } catch (final OrchestraWrapperException e) {
      if (e.getCause() instanceof ProcessNotFoundException) {
        throw (ProcessNotFoundException) e.getCause();
      }
      throw e;
    }
  }

}
