/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import org.ow2.orchestra.env.Environment;
import org.ow2.orchestra.facade.exception.ActivityInstanceNotFoundException;
import org.ow2.orchestra.facade.exception.OrchestraWrapperException;
import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.services.Querier;
import org.ow2.orchestra.services.commands.Command;
import org.ow2.orchestra.util.EnvTool;

public final class GetActivityInstanceCommand implements Command<ActivityInstance> {

  private static final long serialVersionUID = 8856613451037326820L;

  private final ActivityInstanceUUID activityInstanceUUID;

  /**
   * @param activityInstanceUUID
   */
  public GetActivityInstanceCommand(final ActivityInstanceUUID activityInstanceUUID) {
    this.activityInstanceUUID = activityInstanceUUID;
  }
  public ActivityInstance execute(final Environment environment) {
    final Querier querier = EnvTool.getQuerier();
    final ActivityFullInstance activityInstance = querier.getActivityInstance(this.activityInstanceUUID);
    if (activityInstance == null) {
      throw new OrchestraWrapperException(new ActivityInstanceNotFoundException(this.activityInstanceUUID));
    }
    return activityInstance.copy();
  }
}
