/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx.commands;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.env.Environment;
import org.ow2.orchestra.services.commands.Command;
import org.ow2.orchestra.services.itf.Publisher;
import org.ow2.orchestra.services.itf.Repository;
import org.ow2.orchestra.util.Misc;

public final class InitializeCommand implements Command<Object> {

  private static final Logger LOG = Logger.getLogger(InitializeCommand.class.getName());
  private static final long serialVersionUID = 1906399701851091793L;

  public Object execute(final Environment environment) {
    final Repository repository = environment.get(Repository.class);
    final Publisher publisher = environment.get(Publisher.class);
    Misc.fastDynamicLog(InitializeCommand.LOG, Level.INFO, "Orchestra initialization.");
    for (final BpelProcess process : repository.getProcesses()) {
      Misc.fastDynamicLog(InitializeCommand.LOG, Level.INFO, "Redeploying services for process %s", process.getQName());
      publisher.publishServices(process, environment);
    }
    Misc.fastDynamicLog(InitializeCommand.LOG, Level.INFO, "Orchestra initialized.");
    return null;
  }
}
