/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.osgi;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;
import org.ow2.orchestra.osgi.impl.OrchestraExtensionServiceImpl;


/**
 * Base class for registering Orchestra extensions.
 *
 * To register extensions, create a iPOJO component that extends this class.
 * The list of extensions classes are given in the constructor.
 *
 * {@link #start()} is iPOJO validate method
 * {@link #start()} is iPOJO invalidate method
 *
 * Example of metadata.xml for iPOJO component:
 * <component name="org.ow2.orchestra.cluster.jgroups.OrchestraJGroupsActivator"
 *   classname="org.ow2.orchestra.cluster.jgroups.OrchestraJGroupsActivator">
 *   <callback transition="validate" method="start" />
 *   <callback transition="invalidate" method="stop" />
 * </component>
 * <instance component="org.ow2.orchestra.cluster.jgroups.OrchestraJGroupsActivator" />
 *
 * @author Guillaume Porcher
 *
 */
public class ExtensionActivator {
  private ServiceRegistration extensionsRegistration;
  protected final BundleContext context;

  private final Class<?>[] extensions;

  /**
   * @param context
   */
  public ExtensionActivator(final BundleContext context, final Class<?>...extensions) {
    this.context = context;
    this.extensions = extensions;
  }


  public synchronized void start() throws Exception {
    this.extensionsRegistration = this.context.registerService(
        OrchestraExtensionService.class.getName(),
        new OrchestraExtensionServiceImpl(this.extensions), null);
  }

  public synchronized void stop() throws Exception {
    if (this.extensionsRegistration != null) {
      this.extensionsRegistration.unregister();
      this.extensionsRegistration = null;
    }
  }
}
