package org.ow2.orchestra.persistence.db;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.deployment.ActiveProcessHolder;
import org.ow2.orchestra.deployment.ProcessDeploymentCounter;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.runtime.BpelInstance;
import org.ow2.orchestra.runtime.WaitingExecution;
import org.ow2.orchestra.runtime.WaitingExecution.WaitingExecutionIterator;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.PendingMessage;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.services.job.Job;
import org.w3c.dom.Document;

public interface RuntimeDbSession extends DbSession {

  /** searches for the latest version of the given BpelProcess */
  ActiveProcessHolder findActiveBpelProcess(QName processQName);

  /** searches for the all deployed versions of the given BpelProcess */
  List<ProcessDefinitionUUID> findBpelProcesses(QName processQName, boolean lock);

  /** searches for the latest version of the given BpelProcess */
  BpelProcess getBpelProcess(ProcessDefinitionUUID pduuid);

  /** searches for all the BpelProcesses in the database */
  List<ProcessDefinitionUUID> getBpelProcessesUUID();

  /** searches for all the BpelInstances in the database */
  List<BpelInstance> getBpelInstances();

  /** searches for all the BpelInstances in the database */
  BpelInstance getBpelInstance(ProcessInstanceUUID processInstanceUUID);

  /**
   * searches for all the Instances instances of the given BpelProcess in the
   * database
   */
  List<BpelInstance> getBpelInstances(QName processQName);
  List<BpelInstance> getBpelInstances(ProcessDefinitionUUID processUUID);

  /**
   * @return
   */
  ProcessDeploymentCounter getProcessDeploymentCounter();

  WaitingExecutionIterator getWaitingExecutions(
      ReceivingElement receivingElement,
      Map<QName, Document> csProperties);
  WaitingExecution getFirstWaitingExecution(InboundMessageElement inboundMessageElement);

  Collection<PendingMessage> getPendingMessages(OperationKey operationKey);

  /**
   * the jobs for which all the retries have failed and which will not be picked
   * up any more by the jobImpl executor
   */
  List<Job> findJobsWithException();

  /**
   * Force 2nd level cache cleaning for the process state.
   * @param processQName
   */
  void clearActiveProcessCache(QName processQName);
}
