/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.persistence.db.hibernate;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.namespace.QName;

import org.hibernate.FlushMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.db.DbRecorder;
import org.ow2.orchestra.persistence.db.QuerierDbSession;
import org.ow2.orchestra.persistence.db.DbRecorder.Record;

/**
 *
 * @author Gael Lalire
 */
public class QuerierDbSessionImpl extends HibernateDbSession implements QuerierDbSession {

  public QuerierDbSessionImpl(final Session session) {
    this.setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public ActivityFullInstance getActivityInstance(final ActivityInstanceUUID activityInstanceUUID) {
    final Query query = this.getSession().getNamedQuery("findActivityInstanceByUUID");
    query.setString("uuid", activityInstanceUUID.toString());
    return (ActivityFullInstance) query.uniqueResult();
  }

  public ProcessFullInstance getProcessInstance(final ProcessInstanceUUID processInstanceUUID) {
    final Query query = this.getSession().getNamedQuery("findProcessInstanceByUUID");
    query.setString("uuid", processInstanceUUID.toString());
    return (ProcessFullInstance) query.uniqueResult();
  }

  public Set<ActivityFullInstance> findActivityInstance(final ProcessInstanceUUID processUUID) {
    final Query query = this.getSession().getNamedQuery("findActivityInstancesWithProcessUUID");
    query.setString("processInstanceUUID", processUUID.toString());
    final List<ActivityFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance>(results);
    } else {
      return new HashSet<ActivityFullInstance>();
    }
  }

  public Set<ActivityFullInstance> findActivityInstance(final ProcessInstanceUUID processInstanceUUID,
    final ActivityState activityState) {
    final Query query = this.getSession().getNamedQuery("findActivityInstancesWithProcessUUIDAndState");
    query.setString("activityState", activityState.toString());
    query.setString("processInstanceUUID", processInstanceUUID.toString());
    final List<ActivityFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance>(results);
    } else {
      return new HashSet<ActivityFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance() {
    final Query query = this.getSession().getNamedQuery("findProcessInstances");
    final List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(final ProcessDefinitionUUID processDefinitionUUID) {
    final Query query = this.getSession().getNamedQuery("findProcessInstancesWithProcessDefinitionUUID");
    query.setString("processDefinitionUUID", processDefinitionUUID.toString());
    final List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityState processInstanceState) {
    final Query query = this.getSession().getNamedQuery("findProcessInstancesWithProcessDefinitionUUIDAndState");
    query.setString("processInstanceState", processInstanceState.toString());
    query.setString("processDefinitionUUID", processDefinitionUUID.toString());
    final List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public Set<ProcessFullInstance> findProcessInstance(final ActivityState processInstanceState) {
    final Query query = this.getSession().getNamedQuery("findProcessInstancesWithState");
    query.setString("processInstanceState", processInstanceState.toString());
    final List<ProcessFullInstance> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullInstance>(results);
    } else {
      return new HashSet<ProcessFullInstance>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition(final QName name, final ProcessState state) {
    final Query query = this.getSession().getNamedQuery("findProcessDefinitionWithQNameAndState");
    query.setString("name", name.getLocalPart());
    query.setString("nameSpace", name.getNamespaceURI());
    query.setParameter("state", state);

    final List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition(final QName name) {
    final Query query = this.getSession().getNamedQuery("findProcessDefinitionWithQName");
    query.setString("name", name.getLocalPart());
    query.setString("nameSpace", name.getNamespaceURI());

    final List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition(final ProcessState state) {
    final Query query = this.getSession().getNamedQuery("findProcessDefinitionWithState");
    query.setParameter("state", state);

    final List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }

  public List<ProcessFullDefinition> findProcessDefinition() {
    final Query query = this.getSession().getNamedQuery("findProcessDefinition");

    final List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new ArrayList<ProcessFullDefinition>(results);
    } else {
      return new ArrayList<ProcessFullDefinition>();
    }
  }
  public ProcessFullDefinition getProcessDefinition(final ProcessDefinitionUUID processDefinitionUUID) {
    final Query query = this.getSession().getNamedQuery("findProcessDefinitionByUUID");
    query.setString("uuid", processDefinitionUUID.toString());
    return (ProcessFullDefinition) query.uniqueResult();
  }

  public ActivityFullDefinition getActivityDefinition(final ActivityDefinitionUUID activityDefinitionUUID) {
    final Query query = this.getSession().getNamedQuery("findActivityDefinitionByUUID");
    query.setString("uuid", activityDefinitionUUID.toString());
    return (ActivityFullDefinition) query.uniqueResult();
  }

  public Set<ActivityFullDefinition> findActivityDefinitions(final ProcessDefinitionUUID processUUID) {
    final Query query = this.getSession().getNamedQuery("findActivityDefinitions");
    query.setString("uuid", processUUID.toString());
    final List<ActivityFullDefinition> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullDefinition>(results);
    } else {
      return new HashSet<ActivityFullDefinition>();
    }
  }

  public List<Record> getAllRecords(final ProcessInstanceUUID instanceUUID) {
    final Query query = this.getSession().getNamedQuery("findInstanceRecords");
    query.setString("uuid", instanceUUID.toString());
    return query.list();
  }

  public List<Record> getAllActivityRecords(final ActivityInstanceUUID activityInstanceUUID) {
    final Query query = this.getSession().getNamedQuery("findActivityRecords");
    query.setString("uuid", activityInstanceUUID.toString());
    return query.list();
  }

  /**
   * @return
   */
  public Collection<ProcessInstanceUUID> getAllProcessInstanceUUIDsFromRecords() {
    final Query query = this.getSession().getNamedQuery("findRecordedInstanceUUID");
    return query.list();
  }

  /**
   * @param processDefinitionUUID
   * @return
   */
  public Collection<ProcessInstanceUUID> getAllProcessInstanceUUIDsFromRecords(
      final ProcessDefinitionUUID processDefinitionUUID) {
    final Query query = this.getSession().getNamedQuery("findRecordedInstanceUUIDForProcessUUID");
    query.setString("uuid", processDefinitionUUID.toString());
    return query.list();
  }
}
