/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.ow2.orchestra.persistence.tx;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.hibernate.Session;
import org.ow2.orchestra.util.Misc;

/**
 * @author Tom Baeyens
 */
public class HibernateSessionResource implements StandardResource {

  private static final Logger LOG = Logger.getLogger(HibernateSessionResource.class.getName());

  protected Session session;
  protected org.hibernate.Transaction transaction;

  public HibernateSessionResource(final Session session) {
    this.session = session;
    Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.FINEST,
        "beginning transaction on hibernate session %s", System.identityHashCode(session));

    try {
      this.transaction = session.beginTransaction();
    } catch (final RuntimeException e) {
      Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.SEVERE,
          "hibernate transaction begin failed. closing hibernate session: %s", e);
      session.close();
      throw e;
    }

    Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.FINE, "begun hibernate transaction %s on hibernate session %s",
        System.identityHashCode(this.transaction),
        System.identityHashCode(session));
  }

  public void prepare() {
    Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.FINE,
        "flushing hibernate session %s", System.identityHashCode(this.session));
    this.session.flush();
  }

  public void commit() {
    Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.FINE, "committing hibernate transaction %s",
        System.identityHashCode(this.transaction));
    try {
      this.transaction.commit();
    } finally {
      this.closeSession();
    }
  }

  private void closeSession() {
    Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.FINE,
        "closing hibernate session %s", System.identityHashCode(this.session));
    this.session.close();
  }

  public void rollback() {
    Misc.fastDynamicLog(HibernateSessionResource.LOG, Level.FINE,
        "rolling back hibernate transaction %s", System.identityHashCode(this.transaction));
    try {
      this.transaction.rollback();
    } finally {
      this.closeSession();
    }
  }
}
