/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.runtime;

public enum ExecutionState {
  /**
   * between creation of a new process instance and the start of
   * that process instance. The motivation of this state is that variables can
   * be set programmatically on the process instance so that they can be used
   * during initializations of variables and timers
   */
  created,
  /**
   * either executing or in a wait state waiting for a signal. This is the
   * normal state of an execution and the initial state when creating a new
   * execution.
   */
  active,
  /**
   * parents with concurrent child executions are inactive. When an execution
   * has concurrent child executions, it implies that this execution can't be
   * active. For example, at a fork, the parent execution can wait inactively in
   * the fork being till all the child executions are joined. Only leaves of the
   * execution tree can be active.
   */
  inactive,
  /**
   * this execution has ended normally.
   */
  ended,
  /**
   * this execution was cancelled with the {@link #cancel()} method before
   * normal execution ended.
   */
  cancelled,
  /**
   * indicates that this execution is temporary suspended with the
   * {@link #suspend()} method. Human tasks of a suspended execution shouldn't
   * show up in people's task list and timers of suspended executions shouldn't
   * fire and the execution is locked.
   */
  suspended,
  /**
   * indicates that this execution is doing an asynchronous continuation.
   */
  async
}