/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.runtime;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.xml.namespace.QName;

import org.ow2.orchestra.definition.element.Correlation;
import org.ow2.orchestra.definition.element.Correlation.Initiate;
import org.ow2.orchestra.definition.element.InboundMessageElement;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.util.QNameComparator;
import org.ow2.orchestra.util.ReceivingElementUtil;
import org.w3c.dom.Document;


/**
 * @author Guillaume Porcher
 *
 */
public class WaitingExecution {

  public static interface WaitingExecutionIterator {
    WaitingExecution next();
  }

  protected long dbid;
  protected int dbversion;

  // activity the execution is waiting on.
  protected String inboundMessageElementName;


  // execution waiting
  protected BpelExecution bpelExecution;
  // correlationSetRuntimes used by the inbound message element
  protected Set<CorrelationSetRuntime> correlationSetRuntimes;


  // List of correlation keys (one for each receiving element)
  protected List<Integer> correlationKeys = new ArrayList<Integer>();

  protected WaitingExecution() { }

  public WaitingExecution(
      final BpelExecution bpelExecution,
      final InboundMessageElement inboundMessageElement) {
    this.bpelExecution = bpelExecution;
    this.correlationSetRuntimes = new HashSet<CorrelationSetRuntime>();


    // Initialize correlationKeys
    for (final ReceivingElement receivingElement : inboundMessageElement.getReceivingElements()) {
      final SortedMap<QName, Document> csProperties = new TreeMap<QName, Document>(QNameComparator.INSTANCE);
      for (final Correlation correlation : receivingElement.getCorrelations()) {
        final String correlationSetName = correlation.getCorrelationSet().getName();
        final CorrelationSetRuntime csRuntime = bpelExecution.getCorrelationSetByName(correlationSetName);
        if (csRuntime == null) {
          throw new OrchestraRuntimeException("CorrelationSetRuntime " + correlationSetName + " not found");
        }
        if (correlation.getInitiate().equals(Initiate.no) || correlation.getInitiate().equals(Initiate.join)) {
          csProperties.putAll(csRuntime.getPropertyValues());
        }
        this.correlationSetRuntimes.add(csRuntime);
      }
      this.correlationKeys.add(Integer.valueOf(ReceivingElementUtil.getCorrelationPropertiesKey(csProperties)));
    }
    this.inboundMessageElementName = inboundMessageElement.getUUID().toString();

  }

  public BpelExecution getBpelExecution() {
    return this.bpelExecution;
  }
  protected void setBpelExecution(final BpelExecution bpelExecution) {
    this.bpelExecution = bpelExecution;
  }
  public Set<CorrelationSetRuntime> getCorrelationSetRuntimes() {
    return this.correlationSetRuntimes;
  }
  public String getInboundMessageElementName() {
    return this.inboundMessageElementName;
  }

  @Override
  public String toString() {
    return "WaitingExecution[" + this.dbid + "]";
  }
}
