/**
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import java.util.UUID;

import javax.xml.namespace.QName;

import org.ow2.orchestra.env.Environment;
import org.ow2.orchestra.exception.FaultWithMessageVariable;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.services.InvokeExecutor.FinishInvokeCommand;
import org.ow2.orchestra.var.MessageVariable;


/**
 * @author Guillaume Porcher
 *
 */
public class ProcessInstanceMessageCarrier implements MessageCarrier {

  private final long executionId;
  private final String uuid;

  public ProcessInstanceMessageCarrier(final long executionId) {
    this.executionId = executionId;
    this.uuid = UUID.randomUUID().toString();
  }

  public String getUuid() {
    return this.uuid;
  }

  public void setFaultMessage(final QName faultQName, final MessageVariable messageToSend) {
    new FinishInvokeCommand(
        this.executionId,
        new FaultWithMessageVariable(faultQName, messageToSend)
    ).execute(Environment.getCurrent());
  }

  public void setMessage(final MessageVariable messageToSend) {
    new FinishInvokeCommand(
        this.executionId,
        messageToSend
    ).execute(Environment.getCurrent());
  }

  public void throwMissingReply() {
    new FinishInvokeCommand(
        this.executionId,
        new OrchestraRuntimeException("Process or scope is ended but no reply message available")
    ).execute(Environment.getCurrent());
  }

}
