/**
 * Copyright (C) 2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.util;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map.Entry;

import javax.wsdl.Part;
import javax.xml.soap.SOAPMessage;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.var.MessageVariable;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;


/**
 * @author Guillaume Porcher
 *
 */
public final class BpelSOAPUtil {

  private BpelSOAPUtil() { }

  public static SOAPMessage buildRpcSOAPMessage(final String soapAction,
      final MessageVariable message, final String operationName) {
    final Document document = BpelSOAPUtil.buildRpcDocument(message, operationName);
    return SOAPUtil.buildSOAPMessage(document, soapAction);
  }

  public static Document buildRpcDocument(final MessageVariable message, final String operationName) {
    Document document = null;
    try {
      document = XmlUtil.getNewDocument();
      final Element operation = document.createElement(operationName);

      final Iterator<Entry<String, Element>> it = message.getParts().entrySet().iterator();
      while (it.hasNext()) {
        final Entry<String, Element> entry = it.next();
        final String partName = entry.getKey();
        final Element partElement = entry.getValue();

        if (BpelXmlUtil.isDefaultElement(partElement)) {
          final Element partEl = document.createElement(partName);
          partEl.setTextContent(partElement.getTextContent());
          operation.appendChild(partEl);
        } else {
          final Node importedPart = document.importNode(partElement, true);
          operation.appendChild(importedPart);
        }
      }
      document.appendChild(operation);
    } catch (final Exception e) {
      throw new OrchestraRuntimeException("Exception while building Rpc SOAP Message", e);
    }
    return document;
  }

  public static SOAPMessage buildDocumentSOAPMessage(final String soapAction, final MessageVariable message) {
    final Element part = message.getParts().values().iterator().next();
    return SOAPUtil.buildDocumentSOAPMessage(soapAction, part);
  }

  public static MessageVariable buildMessageFromRpcSOAPBodyElement(final Collection<Part> parts, final Element soapBodyElement) {
    final Document messageDoc = XmlUtil.getNewDocument();
    messageDoc.appendChild(messageDoc.createElement("message"));
    for (final Part part : parts) {
      final Element partElt = messageDoc.createElement(part.getName());
      messageDoc.getDocumentElement().appendChild(partElt);

      final Element el = BpelUtil.getPartElement(soapBodyElement, part);
      if (el == null) {
        throw new OrchestraRuntimeException("Impossible to build a Message from the given rpc soap body element."
            + " One element with a localName equals to " + part.getName() + " is mandatory.");
      }
      if (part.getTypeName() != null && BpelUtil.isXmlSchemaNs(part.getTypeName().getNamespaceURI())) {
        //it is a schema type
        String text = null;
        final Node child = el.getFirstChild();
        if (child != null) {
          text = ((Text)child).getData();
        } else {
          text = el.getTextContent();
        }

        final Element elt = BpelXmlUtil.createDefaultElement(messageDoc);
        elt.setTextContent(text);
        partElt.appendChild(elt);
      } else {
        final Node node = messageDoc.importNode(el,true);
        partElt.appendChild(node);
      }
    }
    return new MessageVariable(messageDoc);
  }

  public static MessageVariable buildMessageFromDocumentSOAPBodyElement(
      final Collection<Part> parts,
      final Element soapBodyElement) {
    final Document messageDoc = XmlUtil.getNewDocument();
    messageDoc.appendChild(messageDoc.createElement("message"));
    final Part part = parts.iterator().next();
    final String partName = part.getName();
    final Element partElt = messageDoc.createElement(partName);
    messageDoc.getDocumentElement().appendChild(partElt);
    partElt.appendChild(messageDoc.importNode(soapBodyElement, true));

    return new MessageVariable(messageDoc);
  }

}
